
#include <Adafruit_GPS.h>
#include <SoftwareSerial.h>
#include <TM1650.h>

// Connect the GPS Power pin to 5V
// Connect the GPS Ground pin to ground
// Connect the GPS TX (transmit) pin to Digital 8
// Connect the GPS RX (receive) pin to Digital 7

SoftwareSerial mySerial(8, 7);
SoftwareSerial reyax(5, 4);
Adafruit_GPS GPS(&mySerial);
TM1650 module(A4, A5, 3);  // SDA=A4, SCL=A5

// Set GPSECHO to 'false' to turn off echoing the GPS data to the Serial console
// Set to 'true' if you want to debug and listen to the raw GPS sentences
#define GPSECHO  false
#define KNOTS2MPH 115.077945 // * 100

void displayNumber(int temp, bool hundredths = false)
{
  module.setDisplayDigit(temp / 100, 0, hundredths ? true : false);
  temp = temp - ((temp / 100) * 100);
  module.setDisplayDigit((temp / 10), 1, hundredths ? false : true);
  temp = temp - ((temp / 10) * 10);
  module.setDisplayDigit((temp), 2, false);
}

void setup() {
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW);

  //Display module
  module.clearDisplay();
  delay(1000);  
  
  for(int nVal=0; nVal<=11; nVal++)     
  {
    displayNumber(nVal);
    delay(100);
  }
  
  displayNumber(0);
  
  while (!Serial); // wait for Serial to be ready
  Serial.begin(57600); // The serial port for the Arduino IDE port output
  delay(2000);
  Serial.println("Serial Ready");
  
  reyax.begin(57600);
  delay(2000);  
  Serial.println("Software Serial Reyax Ready");
  
  mySerial.begin(9600);
  delay(2000);
  Serial.println("Software Serial GPS Ready");

  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_10HZ);
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCONLY);
  GPS.sendCommand(PMTK_LOCUS_STARTLOG);
  delay(1000);
}

uint32_t timer = millis();
void loop() 
{
  char c = GPS.read();
  if ((c) && (GPSECHO))
    Serial.write(c);

 if (reyax.available()) {
    Serial.write(reyax.read());
  }
  
  if (GPS.newNMEAreceived()) 
  {
    if (!GPS.parse(GPS.lastNMEA()))   // this also sets the newNMEAreceived() flag to false
    {
      Serial.println(GPS.lastNMEA());
    }
    else
    {
      //Convert knots to Mph * 100
      int gpsSpeed = (int)(GPS.speed * KNOTS2MPH);

      if(gpsSpeed < 50) // 0.50 mph
        displayNumber(gpsSpeed, true);
      else
         displayNumber(gpsSpeed / 10.0);
     
     String mph = String(gpsSpeed);

     //Create AT command message
     String at = "AT+SEND=200," + String(mph.length()) + "," + mph + "\r\n";

     //Output Lat Lon
     Serial.print((GPS.latitude_fixed));
     Serial.write("  ");
     Serial.print((GPS.longitude_fixed));
     Serial.write("  ");
     Serial.print(gpsSpeed);
     Serial.write("\r\n");

     //Write AT Command to Reyax
     reyax.write(at.c_str());
    }
  }
  
  if (Serial.available()) 
  {
    mySerial.write(Serial.read());
  }
}
